@@ -497,23 +497,37 @@ SELECT * FROM f_test_tbl1 ORDER BY c1 DESC;
 
 -- Order by is not null
 EXPLAIN (VERBOSE, COSTS OFF)
-SELECT * FROM f_test_tbl1 ORDER BY c7 IS NOT NULL;
-SELECT * FROM f_test_tbl1 ORDER BY c7 IS NOT NULL;
+SELECT * FROM f_test_tbl1 ORDER BY c7 IS NOT NULL, c1;
+SELECT * FROM f_test_tbl1 ORDER BY c7 IS NOT NULL, c1;
 
 -- Order by is not null desc
 EXPLAIN (VERBOSE, COSTS OFF)
-SELECT * FROM f_test_tbl1 ORDER BY c7 IS NOT NULL DESC;
-SELECT * FROM f_test_tbl1 ORDER BY c7 IS NOT NULL DESC;
+SELECT * FROM f_test_tbl1 ORDER BY c7 IS NOT NULL DESC, c1;
+SELECT * FROM f_test_tbl1 ORDER BY c7 IS NOT NULL DESC, c1;
 
 -- Order by desc nulls first
 EXPLAIN (VERBOSE, COSTS OFF)
-SELECT * FROM f_test_tbl1 ORDER BY c7 DESC NULLS FIRST;
-SELECT * FROM f_test_tbl1 ORDER BY c7 DESC NULLS FIRST;
+SELECT * FROM f_test_tbl1 ORDER BY c7 DESC NULLS FIRST, c1;
+SELECT * FROM f_test_tbl1 ORDER BY c7 DESC NULLS FIRST, c1;
 
 -- Order by asc nulls last
 EXPLAIN (VERBOSE, COSTS OFF)
-SELECT * FROM f_test_tbl1 ORDER BY c7 DESC NULLS LAST;
-SELECT * FROM f_test_tbl1 ORDER BY c7 DESC NULLS LAST;
+SELECT * FROM f_test_tbl1 ORDER BY c7 DESC NULLS LAST, c1;
+SELECT * FROM f_test_tbl1 ORDER BY c7 DESC NULLS LAST, c1;
+
+-- Test LIMIT where ORDER BY is not pushed down due to unsafe pathkeys.
+EXPLAIN (VERBOSE, COSTS OFF)
+SELECT * FROM f_test_tbl1 t1 ORDER BY t1 LIMIT 5;
+SELECT * FROM f_test_tbl1 t1 ORDER BY t1 LIMIT 5;
+
+-- FDW-322: Test mysql_fdw_display_pushdown_list() function.
+SELECT count(*) FROM mysql_fdw_display_pushdown_list();
+
+-- FDW:554 - Array expression should not get pushdown to remote MySQL server as
+-- syntax is not supported.
+EXPLAIN (VERBOSE, COSTS OFF)
+SELECT * FROM f_test_tbl1 t1 WHERE c1 = ANY(ARRAY[100, 200, c1 + 10000]);
+SELECT * FROM f_test_tbl1 t1 WHERE c1 = ANY(ARRAY[100, 200, c1 + 10000]);
 
 -- Cleanup
 DROP TABLE l_test_tbl1;
